
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "../../src/initng.h"


#include <stdio.h>
#include <errno.h>
#include <string.h>
#include "../../src/initng_handler.h"
#include "../../src/initng_global.h"
#include "../../src/initng_plugin_hook.h"


#include "initng_chdir.h"
#include <assert.h>

s_entry CHDIR = { "chdir", STRING, 5, NULL,
    "Change to this directory before launching."
};
int do_chdir(active_h * s, process_h * p __attribute__ ((unused)))
{
    char *tmp = NULL;

    assert(s);
    assert(s->name);
    assert(p);

    D_("do_chdir!\n");
    if (!(tmp = active_db_get_string(&CHDIR, s)))
    {
        D_("CHDIR not set!\n");
        return (TRUE);
    }
    D_("CHDIR TO %s\n", tmp);
    if (chdir(tmp) == -1)
    {
        F_("Chdir failed with %s\n", strerror(errno));
        return (FALSE);
    }
    return (TRUE);
}

int module_init(const char *version)
{
    D_("module_init(CHDIR);\n");
    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module, is compiled for \"%s\" version, and initng is compiled on \"%s\" version, wont load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }
    initng_sdt_add(&CHDIR);
    return (initng_add_hook(A_FORK, 50, &do_chdir));
}

void module_unload(void)
{
    D_("module_unload(CHDIR);\n");
    initng_sdt_del(&CHDIR);
    initng_del_hook(A_FORK, &do_chdir);
}
