/* Initng, a next generatind sysvinit replacement.
 * Copyright (C) 2005 neuron <neuron@hollowtube.mine.nu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"


#include <stdio.h>
#include <stdlib.h>                         /* free() exit() */
#include <string.h>
#include <assert.h>
#include <time.h>

#include "../../src/initng_handler.h"
#include "../../src/initng_global.h"
#include "../../src/initng_plugin_hook.h"
#include "../../src/initng_common.h"
#include "../../src/initng_toolbox.h"
#include "../../src/initng_static_data_id.h"
#include "../../src/initng_static_states.h"
#include "../../src/initng_service.h"

#include "initng_respawn.h"

/* seconds that must pass between respawns */
#ifndef MAX_RESPAWN_RATE
#define MAX_RESPAWN_RATE 10
#endif

#ifndef RESPAWN_SLEEP
#define RESPAWN_SLEEP 1
#endif

static void resp_try(active_h * service);


s_entry RESPAWN = { "respawn", SET, 7, &TYPE_DAEMON,
    "If this is set and dameon dies, it will be restarted."
};
a_state_h WAIT_RESP_TOUT =
    { "WAIT_FOR_RESPAWN_TIMEOUT", IS_DOWN, &resp_try, NULL };

s_entry RESPAWN_PAUSE = { "respawn_pause", INT, 13, &TYPE_DAEMON,
    "Whait this number of seconds, before respawning."
};
s_entry LAST_RESPAWN = { NULL, INT, 0, NULL, NULL };


static int check_respawn(active_h * service, a_state_h * status)
{
    time_t last = 0;

    assert(service);

    /* look for when initng are trying to set a servuce to STOPPED */
    if (status != &STOPPED)
        return (TRUE);

    /* check that current status is RUNNING, so it realy died, and not is manually stopped */
    if (!IS_MARK(service, &RUNNING))
    {
        D_("service %s is gonna be marked STOPPED,\nbut not from status RUNNING, \nits from %s\n\n", service->name, service->a_state->state_name);
        return (TRUE);
    }

    /* make sure the initng is not exiting */
    if (g.sys_state == STATE_STOPPING)
    {
        D_("System state is stopping, wont respawn anything now.\n");
        return (TRUE);
    }

    /* check if the service have respawn enabled */
    if (!active_db_is(&RESPAWN, service))
    {
        D_("Servcice %s dont have RESPAWN flag set, wont respawn!\n");
        return (TRUE);
    }

    /* get times */
    if (active_db_is(&LAST_RESPAWN, service))
        last = (time_t) active_db_get_int(&LAST_RESPAWN, service);
    D_("Now: %i , Last: %i\n", g.now, last);

    /* make sure it wont respawn to ofthen */
    if (last)
    {
        if (g.now - last < MAX_RESPAWN_RATE)
        {
            W_("Wont respawn service %s, it was respawned %i seconds ago.\n",
               service->name, g.now - last);
            return (TRUE);
        }
    }

    /* set the next LAST_RESPAWN no to use. */
    active_db_set_int(&LAST_RESPAWN, service, (int) g.now);

    mark_service(service, &WAIT_RESP_TOUT);
    return (FALSE);
}

static void resp_try(active_h * service)
{
    time_t last = 0;
    int wait = RESPAWN_SLEEP;

    last = (time_t) active_db_get_int(&LAST_RESPAWN, service);

    if (active_db_is(&RESPAWN_PAUSE, service))
        wait = active_db_get_int(&RESPAWN_PAUSE, service);

    if ((g.now - last) < wait)
    {
        D_("Will sleep %i seconds before respawning!\n", wait);
        initng_set_sleep(wait);
        return;
    }

    /* If set to loading, it will automaticly be started. */
    mark_service(service, &LOADING);

    /* to make sure it will be started */
    start_service(service);

    /* don't allow change top STOPPED, it is set to STARTING with start_service abow */
    return;
}


int module_init(const char *version)
{
    D_("module_init();\n");
    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module, is compiled for \"%s\" version, and initng is compiled on \"%s\" version, wont load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }

    initng_sdt_add(&RESPAWN);
    initng_sdt_add(&LAST_RESPAWN);
    initng_add_state(&WAIT_RESP_TOUT);
    initng_add_hook(CONFIRM_ASTATUS_CHANGE, 50, &check_respawn);
    return (TRUE);
}

void module_unload(void)
{
    D_("module_unload();\n");
    initng_sdt_del(&RESPAWN);
    initng_sdt_del(&LAST_RESPAWN);
    initng_del_state(&WAIT_RESP_TOUT);
    initng_del_hook(CONFIRM_ASTATUS_CHANGE, &check_respawn);
}
