
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"

#include <time.h>                           /* time() */
#include <fcntl.h>                          /* fcntl() */
#include <unistd.h>                         /* execv() pipe() usleep() pause() chown() */
#include <sys/wait.h>                       /* waitpid() sa */
#include <sys/ioctl.h>                      /* ioctl() */
#include <stdlib.h>                         /* free() exit() */
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <pwd.h>
#include <sys/stat.h>
#include <assert.h>

#include "../../src/initng_active_db.h"
#include "../../src/initng_global.h"
#include "../../src/initng_plugin_hook.h"
#include "../../src/initng_plugin_callers.h"
#include "../../src/initng_execute.h"
#include "../../src/initng_string_tools.h"
#include "../../src/initng_toolbox.h"
#include "../../src/initng_common.h"
#include "../../src/initng_signal.h"
#include "../../src/initng_fork.h"
#include "../../src/initng_env_variable.h"

#include "initng_simple_launcher.h"

s_entry EXEC = { "exec", VARIABLE_STRINGS, 4, NULL,
    "The path and name for an executable."
};
s_entry EXEC_ARGS = { "exec_args", VARIABLE_STRING, 9, NULL,
    "The arguments for the executable."
};

static int simple_exec(process_h * p, active_h * s, char *e, char *ea);

static int initng_s_launch(active_h * service, process_h * process)
{
    char *e = NULL;
    char *t = NULL;

    assert(service);
    assert(service->name);


    /* WE ARE EXECUTING A START FILE */
    if (active_db_is_var(&EXEC, process->pt->name, service))
    {
        while ((e =
                active_db_get_next_string_var(&EXEC, process->pt->name,
                                              service, e)))
        {

            struct stat test;

            t = fix_variables(e, service);
            /* 11/20/2005 SaTaN0rX: preliminary support for searching the PATH */
            /* only search the PATH if this is NOT already an absolute path */
            if (t[0] != '/')
            {
#define MAX_FILENAME 1000
                char filename[MAX_FILENAME + 3];
                char *tmp;
                int tok_len;
                const char *PATH = getenv("PATH");

                D_("initng_s_launch: %s contains is not absolute path, searching $PATH\n", t);
                if (!PATH)
                {
                    D_("using default path\n");
                    PATH =
                        "/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin";
                }
                D_("PATH determined to be %s\n", PATH);
                /* no absolute path, so search $PATH */
                while (PATH && *PATH)
                {
                    /* fill filename with '\0's, so that all strings are properly terminated. */
                    memset(filename, '\0', MAX_FILENAME + 3);
                    tmp = strchr(PATH, ':');
                    if (tmp)
                        tok_len = tmp - PATH;
                    else
                    {
                        tok_len = strlen(PATH);
                        tmp = (char *) -1;  /* at the end of this loop, PATH will be set to tmp + 1, which is NULL */
                    }
                    if (tok_len >= MAX_FILENAME)
                    {
                        PATH = tmp + 1;
                        continue;           /* PATH segement too long; skip over to next one */
                    }

                    strncpy(filename, PATH, tok_len);
                    if (filename[tok_len - 1] != '/')
                    {
                        filename[tok_len] = '/';
                    }

                    strncat(filename, t, MAX_FILENAME - tok_len);

                    D_("Trying %s\n", filename);
                    if (stat(filename, &test) == -1)
                    {
                        D_("Stat failed in the inner loop (PATH) failed on %s\n", filename);
                    }
                    else if (test.
                             st_mode & (S_IXOTH | S_IXGRP | S_IXUSR | S_ISVTX
                                        | S_ISGID | S_ISUID))
                    {
                        /* filename is an executable */
                        D_("Found %s as Executable\n", filename);
                        /* t is malloced, free it */
                        free(t);
                        /* strdup will call malloc */
                        t = strdup(filename);
                        break;              /* exit loop */
                    }
                    PATH = tmp + 1;
                }
                /* end of loop: either thru break above (in this case, t will have an absolute path pointing
                 * to an executable,
                 * or thru loop end: in this case, t has the same contents as it had before, and stat will fail */
            }
            if (stat(t, &test) == -1)
            {
                F_("Stat failed on %s\n", t);
                free(t);
                continue;

            }

            if (simple_exec(process, service, t,
                            active_db_get_string_var(&EXEC_ARGS,
                                                     process->pt->name,
                                                     service)))
            {
                free(t);
                return (TRUE);
            }
            D_("%s did not work\n", t);
            free(t);
        }
        return (FAIL);
    }

    return (FALSE);
}


static int simple_exec(process_h * process_to_exec, active_h * s,
                       char *e, char *ea)
{

    /* called from inside the service directory, return the PID or 0 on error */

    /* This is the real service kicker */
    pid_t pid_fork;             /* pid got from fork() */
    char **argv = 0;            /* Arguments */
    int argc;                   /* number of arguments */

    assert(process_to_exec);
    assert(s);
    assert(s->name);
    assert(e);

    if ((pid_fork = initng_fork(s, process_to_exec)) == 0)
    {
        char *exec = NULL;
        char *exec_args = NULL;

        ea = fix_variables(ea, s);

        exec = e;
        exec_args = ea;

        D_("FROM_FORK simple_exec(%s,%s);\n", e, ea);

        /* get args from service database */
        if (exec_args)
        {
            D_("FROM_FORK (%s): start args: %s\n", exec, exec_args);

            /*
             * obs!, the split_n, creates a aray, of pointers that points
             * to exec_args, if exec_args is free, argv is usless!
             */
            argv = split_n(exec_args, &argc, 2, 1);

            /* i don't know about this */
            if (!argv[argc - 1])
            {
                argv[argc - 1] = '\0';      /* the last */
            }
            else
            {
                argv[argc] = '\0';          /* must be null */
            }
        }
        else
        {
            argv = initng_calloc(2, sizeof(char *));    /* alloc a null */
            argv[1] = '\0';                 /* set to null */
        }

        /* set argv[0] */
        argv[0] = exec;


        {
            int i;

            for (i = 0; argv[i]; i++)
                D_("i.e.f.a.e.(): argv[%i]=\"%s\"\n", i, argv[i]);
        }


        /* FINALLY EXECUTE *//* execve replaces the running process */
        execve(argv[0], argv, new_environ(s));

        /* Will never get here if execve succeeded */
        F_("EEEEEEEEEEEEEEEEEEERRRRRRRRRRRRRRRRRRRRRRRRRRROOOOOOOOOOOOOOOOOOOOOOOOOORRRRRRRRRRRRRRR!!!!!!!!!\n");
        F_("Cant execute source %s, \"%s\" !\n", e, ea);

        /* free variable, good for making valgrind quit */
        /*free(argv[0]);
           free(argv);
           free(exec_args); */

        /* clean up */
        /*initng_free(); */

        /* exit 1, to mark a failure */
        _exit(1);
    }

    /* save pid of fork */
    D_("FROM_FORK Forkstarted pid %i.\n", pid_fork);

    if (pid_fork > 0)
    {
        process_to_exec->pid = pid_fork;
        return (TRUE);
    }

    process_to_exec->pid = 0;
    return (FALSE);
    /* if to test want to lock this up util fork is done ...   waitpid(pid_fork,0,0); */
}                                           /* end fork_and_exec() */


int module_init(const char *version)
{
    D_("initng_simple_plugin: module_init();\n");
    /*
       if (strcmp(version, INITNG_VERSION) != 0)
       {
       F_("This module, is compiled for \"%s\" version, and initng is compiled on \"%s\" version, wont load this module!\n", INITNG_VERSION, version);
       return (FALSE);
       } */

    initng_add_hook(LAUNCH, 40, &initng_s_launch);
    initng_sdt_add(&EXEC);
    initng_sdt_add(&EXEC_ARGS);
    return (TRUE);
}

void module_unload(void)
{
    D_("initng_simple_plugin: module_unload();\n");

    initng_sdt_del(&EXEC);
    initng_sdt_del(&EXEC_ARGS);
    initng_del_hook(LAUNCH, &initng_s_launch);

}
