
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "initng.h"
#include "initng_global.h"
#include "initng_load_module.h"
#include "initng_toolbox.h"
#include "initng_static_states.h"
#include "initng_static_process_types.h"
#include "initng_handler.h"
#include "initng_service.h"

/* HERE IS THE GLOBAL DEFINED STRUCT, WE OFTHEN RELATE TO */
s_global g;

void initng_set_runlevel(const char *runlevel)
{
    /* first free the old_runlevel */
    if (g.old_runlevel)
    {
        free(g.old_runlevel);
        g.old_runlevel = NULL;
    }

    /* then move the last new, to the old */
    if (g.runlevel)
    {
        g.old_runlevel = g.runlevel;
        g.runlevel = NULL;
    }

    /* and set the new */
    g.runlevel = i_strdup(runlevel);
}

void initng_new(int argc, char *argv[], char *env[])
{
    int i;

    assert(argv);
    assert(env);

    /* zero the complete s_global */
    memset(&g, 0, sizeof(s_global));

    /* we want to keep a copy of the arguments passed to us, this will be overwritten by set_title() */
    g.Argc = argc;
    g.Argv0 = argv[0];
    g.Argv = (char **) initng_calloc(argc + 1, sizeof(char *));
    assert(g.Argv);

    for (i = 0; i < argc; i++)
    {
        g.Argv[i] = i_strdup(argv[i]);
        assert(g.Argv[i]);
    }
    g.Argv[argc] = NULL;

    /* 
     * We want to change our process name, because it looks nice in 
     * "ps" output. But we can only use space that the kernel 
     * allocated for us when we started. This is argv[] and env[] 
     * together. 
     */
    g.maxproclen = 0;
    for (i = 0; i < argc; i++)
    {
        g.maxproclen += strlen(argv[i]) + 1;
    }
    for (i = 0; env[i] != NULL; i++)
    {
        g.maxproclen += strlen(env[i]) + 1;
    }
    D_("Maximum length for our process name is %d\n", g.maxproclen);

    /*
     * initziate all databases, next and prev have to point to own struct
     */
    INIT_LIST_HEAD(&g.active_database.list);
    INIT_LIST_HEAD(&g.service_cache.list);
    INIT_LIST_HEAD(&g.states.list);
    INIT_LIST_HEAD(&g.ptypes.list);
    INIT_LIST_HEAD(&g.call_db.list);
    INIT_LIST_HEAD(&g.module_db.list);
    INIT_LIST_HEAD(&g.option_db.list);
    INIT_LIST_HEAD(&g.command_db.list);
    INIT_LIST_HEAD(&g.stypes.list);


    /*
     * default global variables - clearesed by memset abowe
     * g.interrupt = FALSE;
     * g.sys_state = STATE_NULL;
     * g.runlevel = NULL;
     * g.dev_console = NULL;
     * g.modules_to_unload = FALSE;
     * g.hot_reload = FALSE;
     */

    /* Add defaults (static) to data_types */
    initng_sdt_add_defaults();
    /* Add static stats to g.states */
    add_static_states();
    /* Add static ptypes */
    add_static_ptypes();
    /* Add static stypes */
    initng_add_static_stypes();

    /* CLEARED by memset abow 
       #ifdef DEBUG
       g.verbose = 0;
       {
       int i;

       for (i = 0; i < MAX_VERBOSES; i++)
       g.verbose_this[i] = NULL;
       }
       #endif
     */
}



void initng_parse_argv(char **argv)
{
    int i;

    for (i = 0; argv[i]; i++)
    {
        /* set opt to argv */
        char *opt = argv[i];

        /* don't parse options starting with an '+' */
        if (opt[0] == '+')
            continue;
        /*
         * if arg is --verbose, skip the two -- here, and start checking.
         * if arg is -verbose, skip and continue for loop.
         * if arg is verbose, check it below
         */

        /* if its a double --, its ok */
        if (opt[0] == '-' && opt[1] == '-')
            opt += 2;
        /* but a single is not ! */
        if (opt[0] == '-')
            continue;


#define R_L "runlevel="
#define KR_L "runlevel:"
#define V_T "verbose_add="
#define C_L "console="

#ifdef DEBUG
        if (strcmp(opt, "verbose") == 0)
            g.verbose = TRUE;
#endif
        if (strcmp(opt, "i_am_init") == 0)
            g.i_am_init = TRUE;
        if (strcmp(opt, "hot_reload") == 0)
        {
            W_(" Will start after a hot reload ...\n");
            g.hot_reload = TRUE;
        }
        if (strcmp(opt, "i_am_not_init") == 0)
            g.i_am_init = FALSE;

        if (strcmp(opt, "no_circular") == 0)
            g.no_circular = TRUE;

        /* the ones with a value */
        if (strncmp(opt, R_L, strlen(R_L)) == 0)
            g.runlevel = i_strdup((opt) + strlen(R_L));
        if (!g.runlevel && strncmp(opt, KR_L, strlen(KR_L)) == 0)
            g.runlevel = i_strdup((opt) + strlen(KR_L));
        if (strncmp(opt, C_L, strlen(C_L)) == 0)
            g.dev_console = i_strdup((opt) + strlen(C_L));
#ifdef DEBUG
        if (strncmp(opt, V_T, strlen(V_T)) == 0)
            verbose_add((opt) + strlen(V_T));
#endif
    }
}

void initng_free(void)
{
    int i;

    /* free all databases */
    active_db_free_all();                   /* clean process_db */
    service_db_free_all();                  /* clean service_cache_db */
    initng_sdt_free_all();                  /* clean option_db */
    initng_command_free_all();              /* clean command_db */

    /* free runlevelname string */
    if (g.runlevel)
        free(g.runlevel);

    /* free console string if set */
    if (g.dev_console)
        free(g.dev_console);

    /* free ur copy of argv, and argc */
    for (i = 0; i < g.Argc; i++)
    {
        free(g.Argv[i]);
    }
    free(g.Argv);


    /* close all open fd, over 3, this may hang boot */
    for (i = 3; i <= 1013; i++)
    {
        close(i);
    }

    /* zero the complete s_global, so there migt be no value left pointing in here */
    memset(&g, 0, sizeof(s_global));
}
