
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "initng.h"

#include <time.h>                           /* time() */
#include <fcntl.h>                          /* fcntl() */
#include <sys/un.h>                         /* memmove() strcmp() */
#include <sys/wait.h>                       /* waitpid() sa */
#include <linux/kd.h>                       /* KDSIGACCEPT */
#include <sys/ioctl.h>                      /* ioctl() */
#include <stdio.h>                          /* printf() */
#include <stdlib.h>                         /* free() exit() */
#include <sys/reboot.h>                     /* reboot() RB_DISABLE_CAD */
#include <assert.h>

#include "initng_global.h"
#include "initng_handler.h"
#include "initng_process_db.h"
#include "initng_execute.h"
#include "initng_toolbox.h"
#include "initng_main.h"
#include "initng_common.h"

#include "main.h"
#include "initng_service_cache.h"
#include "initng_active_db.h"
#include "initng_load_module.h"
#include "initng_plugin_callers.h"
#include "initng_fd.h"

#include "initng_kill_handler.h"
#include "initng_static_data_id.h"
#include "initng_static_states.h"

/* called when a process got killed, identify it, and make a call with a pointer to the process */
void handle_killed_by_pid(pid_t kpid, int r_code)
{
    /* The process that got killed */
    active_h *service = NULL;
    process_h *process, *safe = NULL;

    D_("handle_killed_by_pid(%i);\n", kpid);

    /* don't do anything on this pid */
    if (kpid <= 1)
        return;

    /* Look in process database for a match */
    if (!(service = active_db_find_by_pid(kpid)))
    {
        D_("handle_killed_by_pid(%i): No match!\n", kpid);
        return;
    }

    /* check service state */
    if (!service->a_state)
    {
        W_("Sertive has no state!\n");
        return;
    }

    D_("handle_killed_by_pid(%i): found service \"%s\"...\n", kpid,
       service->name);

    while_processes_safe(process, safe, service)
    {
        /* make sure thats the right process */
        if (process->pid != kpid)
            continue;

        /* set r_code */
        process->r_code = r_code;

        /* Check if a plugin wants to override handle_killed behavior */
        if (initng_plugin_handle_killed(service, process))
        {
            D_("Plugin did handle this kill.\n");
            return;
        }

        if (process->pt && process->pt->kill_handler)
        {
            (*process->pt->kill_handler) (service, process);
        }
        else
        {
            F_("process %i died with unknown handler, freeing process!\n",
               kpid);
            list_del(&process->list);
            process_db_free(process);
        }
        return;
    }

    F_("handle_killed_by_pid(%i): traced killed pid to a service but can't find servicetype!\n", kpid);
    return;
}
